package com.ruoyi.system.service.impl;

import com.alibaba.nacos.shaded.com.google.common.collect.Lists;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.utils.bean.BeanUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.security.service.TokenService;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.domain.SnCard;
import com.ruoyi.system.enums.BussLogEnum;
import com.ruoyi.system.mapper.SnCardMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ISnCardService;
import com.ruoyi.system.service.ISysBussLogService;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class SnCardServiceImpl implements ISnCardService {
   @Autowired
   private SnCardMapper snCardMapper;
   @Autowired
   private SysUserMapper userMapper;
   @Autowired
   private TokenService tokenService;
   @Autowired
   private ISysBussLogService sysBussLogService;

   public AjaxResult charge(SnCard snCard) {
      SysUser user = this.userMapper.selectUserByUserName(snCard.getUserName());
      if (user == null) {
         return AjaxResult.error("商户不存在");
      } else if (!SecurityUtils.matchesPassword(snCard.getPassword(), user.getPassword())) {
         return AjaxResult.error("商户账户密码有误");
      } else if (!Objects.equals(user.getUserType(), "common")) {
         return AjaxResult.error("该账户不能充值卡密");
      } else {
         SnCard param = new SnCard();
         param.setCode(snCard.getCode());
         List<SnCard> list = this.snCardMapper.selectSnCardList(param);
         if (list.isEmpty()) {
            return AjaxResult.error("卡密无效");
         } else {
            SnCard card = (SnCard)list.get(0);
            if (card.getStatus() != 0) {
               return AjaxResult.error("卡密已被使用");
            } else {
               SnCard up = new SnCard();
               up.setCardId(card.getCardId());
               up.setStatus(1);
               up.setUserId(user.getUserId());
               up.setUserName(user.getUserName());
               up.setAgentId(user.getAgentId());
               up.setAgentName(user.getAgentName());
               up.setUseTime(DateUtils.getNowDate());
               this.snCardMapper.updateSnCard(up);
               SysUser upUser = new SysUser();
               upUser.setUserId(user.getUserId());
               Calendar calendar = Calendar.getInstance();
               if (user.getPeriod() != null && user.getPeriod().after(new Date())) {
                  calendar.setTime(user.getPeriod());
               }

               calendar.add(5, card.getDay());
               Date newDate = calendar.getTime();
               upUser.setPeriod(newDate);
               this.userMapper.updateUser(upUser);
               this.sysBussLogService.insertSysBussLog(BussLogEnum.ChargeCard.getType(), user.getUserId(), user.getUserName(), "卡密充值 " + card.getDay() + " 天\n\n到期时间" + DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, newDate) + "\n\n使用卡密\n" + card.getCode());
               return AjaxResult.success("充值成功");
            }
         }
      }
   }

   public SnCard selectSnCardByCardId(Long cardId) {
      SnCard obj = this.snCardMapper.selectSnCardByCardId(cardId);
      if (obj == null) {
         return null;
      } else {
         LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
         if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
            if (loginUser.getRoles().contains("dealer")) {
               return obj.getDealerId().equals(loginUser.getUserid()) ? obj : null;
            } else if (loginUser.getRoles().contains("agent")) {
               return obj.getAgentId().equals(loginUser.getUserid()) ? obj : null;
            } else {
               return obj.getUserId().equals(loginUser.getUserid()) ? obj : null;
            }
         } else {
            return obj;
         }
      }
   }

   public List<SnCard> selectSnCardList(SnCard obj) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (loginUser.getRoles().contains("dealer")) {
         obj.setDealerId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("agent")) {
         obj.setAgentId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("common")) {
         obj.setUserId(loginUser.getUserid());
      }

      return this.snCardMapper.selectSnCardList(obj);
   }

   public AjaxResult insertSnCard(SnCard snCard) {
      SysUser user = this.userMapper.selectUserById(snCard.getDealerId());
      if (user == null) {
         return AjaxResult.error("未查询到经销商信息");
      } else {
         LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
         snCard.setDay(snCard.getType());
         snCard.setStatus(0);
         snCard.setDealerId(user.getUserId());
         snCard.setDealerName(user.getUserName());
         snCard.setCreateId(loginUser.getUserid());
         snCard.setCreateName(loginUser.getUsername());
         snCard.setCreateTime(DateUtils.getNowDate());
         List<SnCard> cards = new ArrayList();

         for(int i = 0; i < snCard.getNumber(); ++i) {
            SnCard obj = (SnCard)BeanUtils.copy(snCard, SnCard.class);
            obj.setCode(UUID.randomUUID().toString());
            cards.add(obj);
         }

         Lists.partition(cards, 3000).forEach((tmpList) -> {
            this.snCardMapper.insertSnCards(tmpList);
         });
         return AjaxResult.success();
      }
   }

   public int updateSnCard(SnCard snCard) {
      snCard.setUpdateTime(DateUtils.getNowDate());
      return this.snCardMapper.updateSnCard(snCard);
   }

   public int deleteSnCardByCardIds(Long[] cardIds) {
      return this.snCardMapper.deleteSnCardByCardIds(cardIds);
   }

   public int deleteSnCardByCardId(Long cardId) {
      return this.snCardMapper.deleteSnCardByCardId(cardId);
   }
}
