package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.security.service.TokenService;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.domain.SnServerTemplate;
import com.ruoyi.system.domain.SnTemplate;
import com.ruoyi.system.mapper.SnServerTemplateMapper;
import com.ruoyi.system.mapper.SnTemplateMapper;
import com.ruoyi.system.service.ISnTemplateService;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class SnTemplateServiceImpl implements ISnTemplateService {
   @Autowired
   private SnTemplateMapper snTemplateMapper;
   @Autowired
   private SnServerTemplateMapper snServerTemplateMapper;
   @Autowired
   private TokenService tokenService;

   public SnTemplate selectSnTemplateByTemplateId(Long templateId) {
      return this.snTemplateMapper.selectSnTemplateByTemplateId(templateId);
   }

   public List<SnTemplate> selectSnTemplateList(SnTemplate snTemplate) {
      snTemplate.setIsDelete(0);
      return this.snTemplateMapper.selectSnTemplateList(snTemplate);
   }

   public int insertSnTemplate(SnTemplate snTemplate) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      snTemplate.setIsDelete(0);
      snTemplate.setCreateId(loginUser.getUserid());
      snTemplate.setCreateName(loginUser.getUsername());
      snTemplate.setCreateTime(DateUtils.getNowDate());
      snTemplate.setAgentId(Long.parseLong(loginUser.getSysUser().getCreateBy()));
      snTemplate.setAgentName(loginUser.getSysUser().getCreateName());
      return this.snTemplateMapper.insertSnTemplate(snTemplate);
   }

   public int updateSnTemplate(SnTemplate snTemplate) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      snTemplate.setUpdateId(loginUser.getUserid());
      snTemplate.setUpdateName(loginUser.getUsername());
      snTemplate.setUpdateTime(DateUtils.getNowDate());
      return this.snTemplateMapper.updateSnTemplate(snTemplate);
   }

   public AjaxResult deleteSnTemplateByTemplateIds(Long[] templateIds) {
      SnServerTemplate param = new SnServerTemplate();
      param.getParams().put("serverTemplateIds", templateIds);
      param.setIsDelete(0);
      List<SnServerTemplate> templates = this.snServerTemplateMapper.selectSnServerTemplateList(param);
      if (!templates.isEmpty()) {
         String ips = (String)templates.stream().map(SnServerTemplate::getServerIp).collect(Collectors.joining(" , "));
         return AjaxResult.error("服务器【" + ips + "】正在使用模板，不能删除");
      } else {
         int num = this.snTemplateMapper.deleteSnTemplateByTemplateIds(templateIds);
         return AjaxResult.success(num);
      }
   }

   public AjaxResult countryNum() {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      Long userId = null;
      if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
         userId = loginUser.getUserid();
      }

      List<Map<String, Object>> list = this.snTemplateMapper.countryNum(userId);
      Map<String, Long> map = new HashMap();
      list.forEach((tmp) -> {
         map.put((String)tmp.get("country"), (Long)tmp.get("count"));
      });
      return AjaxResult.success(map);
   }
}
